
# q-expansion of newform 722.4.a.e, downloaded from the LMFDB on 29 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 722
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[2], [5], [3], [-32], [4], [-69], [19], [0], [67], [51], [-132], [-14], [-413], [129], [-617], [383], [-599], [-217], [-225], [701], [1015], [349], [-592], [-1349], [-613], [1235], [1632], [2060], [1055], [1006], [1987], [-1803], [-669], [-2725], [71], [-656], [-1053], [68], [-921], [-1893], [-20], [1839], [2992], [-1393], [-4214], [767], [2365], [-2439], [-1708], [4618], [3219], [-4236], [-2429], [243], [1047], [1905], [-5833], [-3727], [-5294], [1375], [-1307], [3818], [873], [4180], [-3165], [4539], [3660], [-5173], [-8591], [6946], [-8226], [11385], [-2953], [5006], [6764], [-4963], [3623], [-7629], [-9117], [4935], [6516], [10915], [-5509], [-3753], [7813], [-6117], [-7146], [-18118], [16935], [-7788], [12500], [-12105], [296], [3213], [3017], [-7277], [-10693], [-7190], [8647], [4043], [-5389], [6727], [-19908], [-8730], [-4732], [-23882], [-16547], [-15029], [13787], [11382], [-25312], [-23497], [-13221], [15316], [-6049], [-24089], [1633], [14592], [-23430], [4135], [-8285], [23990], [690], [5760], [4348], [-10289], [-30745], [841], [-271], [-8102], [15649], [-10335], [-14669], [-41437], [-34258], [6855], [-29457], [20716], [3722], [11358], [8479], [13371], [-26585], [36603], [31238], [30657], [4443], [-6969], [39319], [9380], [-10249], [-28698], [31779], [3567], [-31079], [32856], [-7736], [30519], [9407], [26815], [17449], [-39873], [-11979], [18201], [37398], [44037], [9829], [39983], [53755], [-23289], [22380], [2383], [-32319], [9022], [36355], [-5529], [-26889], [-52441], [53496], [-9089], [-10793], [-37796], [-52949], [21059], [-47848], [34734], [51155], [22213], [5926], [-28312], [-7341], [46491], [-29156], [30207], [72627], [-68577], [39895], [-73301], [80246], [79224], [-21993], [-38033], [-23841], [43171], [-59451], [-48201], [-54073], [48399], [-33105], [-27496], [-20365], [-24685], [-56700], [66599], [15688], [24795], [10961], [11838], [-3392], [28703], [-42853], [70984], [-60329], [-74455], [-45295], [13127], [-18426], [-35671], [-56187], [-39492], [-54109], [-60533], [5336], [34686], [22800], [56209], [-18226], [60186], [44761], [-5681], [2773], [27604], [39739], [-15945], [50695], [-50272], [-23749], [89127], [-95839], [44187], [36274], [-28665], [-27384], [73519], [79859], [83187], [46299], [7628], [2499], [-28993], [-83777], [58485], [130371], [-5993], [41083], [-41884], [19574], [-55638], [72747], [-131505], [-71134], [-116316], [-99797], [15712], [-90553], [-78163], [-67916], [-35749], [-13213], [36391], [-29628], [145916], [46479], [109098], [141637], [75963], [76478], [-46993], [54997], [-143770], [-41850], [-135208], [491], [93972], [-47573], [-72510], [71959], [-7225], [-135095], [109940], [86467], [19331], [57867], [-42044], [43441], [55], [-107609], [-35233], [168699], [-8078], [173228], [129395], [113727], [-123716], [-41991], [120190], [42863], [-9063], [-121565], [-61522], [-101143], [-15221], [160803], [-111309], [283], [30103], [-159533], [157780], [64737], [154083], [56107], [-137297], [121509], [67932], [-11745], [-127377], [49434], [-212318], [-4986], [142816], [-20873], [6119], [-14061], [113160], [159431], [128745], [59983], [137291], [109221], [-65222], [14983], [-162149], [-168117], [-197784], [55627], [-192066], [-27363], [-46755], [106987], [55731], [-29069], [-197865], [83259], [16256], [-40675], [207471], [-19050], [196097], [-215961], [145747], [12980], [-50063], [71983], [-82923], [93930], [-154379], [53704], [37823], [-45617], [66615], [86586], [212542], [141783], [213743], [43160], [-28383], [-238096], [-272769], [-199121], [-17004], [28575], [220227], [65772], [-84049], [19443], [73471], [9629], [95134], [275189], [-237621], [261515], [-152797], [82552], [-267757], [-58133], [-56671], [-8421], [-89538], [-82801], [-94088], [-93013], [-126317], [-293568], [129200], [168906], [37210], [200151], [118280], [-85333], [215147], [-112869], [251755], [-116816], [135799], [-154202], [138294], [-220692], [-69565], [22252], [180165]]
